#ifndef __RT_LIB_RT_FilterBase_h__
#define __RT_LIB_RT_FilterBase_h__

#include <fire/private/prefix.h>

#include <cassert>

#include <memory>
#include <vector>
#include <typeinfo>

namespace FIRE_NAMESPACE {

struct RT_RequestRep;
class RT_Message;
class RT_Record;
class RT_Consumer;
struct RT_ConsumerImpl;

class RT_FilterBase
{
protected:
  RT_FilterBase (RT_ConsumerImpl &cons) : m_consumer(&cons), m_on_off(true) {}

public:
  virtual ~RT_FilterBase() {}

  //  to register a new request
  virtual bool on_request( int srv_tag, const RT_RequestRep& req );;
  //  decides to pass a message through the filter
  virtual bool filter_off( int srv_tag, const RT_Message& msg );

  //  to handles a control message
  virtual void on_control( bool is_connected, RT_Message& msg );
  //  to handle an update message, returns 'true' if the msg was updated
  virtual bool on_message( int srv_tag, RT_Message& msg, RT_Record& rec ) = 0;
  //  to handle a cancellation of a srv_tag
  virtual void on_cancel( int srv_tag ) = 0;

  //  to start/stop processing
  virtual bool start() { m_on_off = true; return true; }
  //  a command to stop processing messages
  virtual bool stop() { m_on_off = false; return true; }
  bool is_active() const { return m_on_off; }

  virtual const std::type_info& feature_type() const = 0;

  virtual void produce_msg( RT_Message& msg );

protected:
  const RT_ConsumerImpl* Consumer() const { return m_consumer; }
  RT_ConsumerImpl& Consumer() { assert(m_consumer); return *m_consumer; }
  void Consumer(RT_ConsumerImpl* val) { m_consumer = val; }

private:
  RT_ConsumerImpl* m_consumer;
  bool m_on_off;  //  enable/disable flag
};

typedef std::auto_ptr<RT_FilterBase> filter_ptr_type;

//////////////////////////////////////////////////////////////////////////
//  implementation
//
inline bool RT_FilterBase::on_request( int /*srv_tag*/, const RT_RequestRep& /*req*/) {
  return true;
}

inline bool RT_FilterBase::filter_off( int /*srv_tag*/, const RT_Message& /*msg*/) {
  return true;
}

inline void RT_FilterBase::on_control( bool /*is_connected*/, RT_Message& /*msg*/)
{}

} //  namespace FIRE_NAMESPACE

#endif // __RT_LIB_RT_FilterBase_h__
