#ifndef __FIRE_PREFIX_H__
#error "Please do not include this file directly, use prefix.h"
#endif

#if !(defined(_WIN32) || defined(__WIN32__) || defined(WIN32))
#error "This is for Win32 only"
#endif

/*
 * Lets do some error checking first
 */
#ifndef _MSC_VER
#error "Unknown compiler, fdsrt only supports Microsoft C++ compiler"
#endif

/* Disable overly verbose compiler warnings */
#pragma warning(disable: 4355)  /* used in base member initializer list */

/* MSVC++ 10.0 _MSC_VER == 1600 (Visual Studio 2010)
   MSVC++ 11.0 _MSC_VER == 1700 (Visual Studio 2012)
   MSVC++ 12.0 _MSC_VER == 1800 (Visual Studio 2013)
   MSVC++ 14.0 _MSC_VER == 1900 (Visual Studio 2015)
   MSVC++ 14.1  _MSC_VER == 1910 (Visual Studio 2017 version 15.0)
   MSVC++ 14.11 _MSC_VER == 1911 (Visual Studio 2017 version 15.3)
   MSVC++ 14.12 _MSC_VER == 1912 (Visual Studio 2017 version 15.5)
   MSVC++ 14.13 _MSC_VER == 1913 (Visual Studio 2017 version 15.6)
   MSVC++ 14.14 _MSC_VER == 1914 (Visual Studio 2017 version 15.7)
   MSVC++ 14.15 _MSC_VER == 1915 (Visual Studio 2017 version 15.8)
   MSVC++ 14.16 _MSC_VER == 1916 (Visual Studio 2017 version 15.9)
  */
#ifdef RT_TOOLKIT
#if (_MSC_VER > 1916) || (_MSC_VER < 1700) || (_MSC_VER == 1800)
#pragma message("Microsoft compiler version = " FIRE_STRINGIZE(_MSC_VER))
#error "Microsoft Bad Compiler Version, fdsrt supports VS11.0, 14.0 to 14.16; cl version 16.0, 17.0, 19.0 respectively"
#endif
#else
#if (_MSC_VER > 1916) || (_MSC_VER < 1700)
#pragma message("Microsoft compiler version = " FIRE_STRINGIZE(_MSC_VER))
#error "Microsoft Bad Compiler Version, fire requires VS11.0, 14.0 to 14.16; cl version 16.0, 17.0, 19.0"
#endif
#endif


#if defined(__MSVC_RUNTIME_CHECKS) && !defined(_DEBUG)
#pragma message("/RTC option will lead to linker errors in non-debug builds")
#error "Please use the /MDd switch (multi-threaded Debug DLL C-runtime)"
#endif

/*
 *  With the new interface design, this check is not needed :)
 *  #if !defined(_MT) || !defined(_DLL)
 *  #ifdef _DEBUG
 *  #error "Please use the /MDd switch (multi-threaded Debug DLL C-runtime)"
 *  #else
 *  #error "Please use the /MD switch (multi-threaded DLL C-runtime)"
 *  #endif
 *  #endif
 */

#if (_MSC_VER == 1700)
#define FIRE_MSC_VERSION "vc110"
#elif (_MSC_VER == 1900)
#define FIRE_MSC_VERSION "vc140"
#elif (_MSC_VER >= 1910 && _MSC_VER <= 1916)
#define FIRE_MSC_VERSION "vc141"
#endif
#ifdef _WIN64
#define FIRE_X64_APPEND "-x64"
#else
#define FIRE_X64_APPEND ""
#endif

#ifdef _WIN64
#define FIRE_ADDRESS_WIDTH "win64"
#else
#define FIRE_ADDRESS_WIDTH "win32"
#endif

#ifdef FIRE_EXPORTS
#ifdef _USRDLL
#define FIRE_DYNAMIC_FLAG "d"
#else
#define FIRE_DYNAMIC_FLAG "s"
#endif
#else
#ifndef FIRE_LINK_STATIC
#define FIRE_DYNAMIC_FLAG "d"
#else
#define FIRE_DYNAMIC_FLAG "s"
#endif
#endif

#ifdef _MT
#ifdef _DLL
#define FIRE_RUNTIME_FLAG "MD"
#else
#define FIRE_RUNTIME_FLAG "MT"
#endif
#else
#error "Please use either the /MD(d) or /MT(d) switch"
#endif

#ifdef _DEBUG
#define FIRE_DEBUG_FLAG "d"
#else
#define FIRE_DEBUG_FLAG
#endif

#ifndef RT_TOOLKIT
#define FIRE_RT_PLATFORM_CHECK FIRE_MSC_VERSION "_x86_" FIRE_ADDRESS_WIDTH FIRE_DYNAMIC_FLAG FIRE_RUNTIME_FLAG FIRE_DEBUG_FLAG
#define FIRE_LIB_NAME "fire_" FIRE_STRINGIZE(FIRE_MAJOR_VERSION) "_" FIRE_STRINGIZE(FIRE_INCLUDE_VERSION) "_" FIRE_RT_PLATFORM_CHECK ".lib"
#else
#define FIRE_RT_PLATFORM_CHECK FIRE_MSC_VERSION "_x86_" FIRE_ADDRESS_WIDTH FIRE_DEBUG_FLAG
#define FIRE_LIB_NAME "fdsrt_" FIRE_RT_PLATFORM_CHECK ".lib"
#endif

#if (_MSC_VER >= 1900) && (_MSC_VER <= 1916)
#define CRYPTOPP_LIB_NAME "crypto++-" FIRE_MSC_VERSION "-" FIRE_RUNTIME_FLAG FIRE_DEBUG_FLAG ".lib"
#define OPENSSL_LIBEAY_NAME "libeay32-s-" FIRE_RUNTIME_FLAG FIRE_DEBUG_FLAG ".lib"
#define OPENSSL_SSLEAY_NAME "ssleay32-s-" FIRE_RUNTIME_FLAG FIRE_DEBUG_FLAG ".lib"
#define CURL_LIB_NAME "libcurl-" FIRE_MSC_VERSION "-" FIRE_RUNTIME_FLAG FIRE_DEBUG_FLAG ".lib"
#define ZLIB_NAME "zlib-" FIRE_MSC_VERSION FIRE_X64_APPEND "-" FIRE_RUNTIME_FLAG FIRE_DEBUG_FLAG ".lib"
#else
#define CRYPTOPP_LIB_NAME "cryptlib_5_6_2_" FIRE_MSC_VERSION "_x86_" FIRE_ADDRESS_WIDTH "s" FIRE_RUNTIME_FLAG FIRE_DEBUG_FLAG ".lib"
#define OPENSSL_LIBEAY_NAME "openssl_libeay_1_0_1_h_" FIRE_MSC_VERSION "_x86_" FIRE_ADDRESS_WIDTH "s" FIRE_RUNTIME_FLAG FIRE_DEBUG_FLAG ".lib"
#define OPENSSL_SSLEAY_NAME "openssl_ssleay_1_0_1_h_" FIRE_MSC_VERSION "_x86_" FIRE_ADDRESS_WIDTH "s" FIRE_RUNTIME_FLAG FIRE_DEBUG_FLAG ".lib"
#define CURL_LIB_NAME "libcurl_7_21_7_" FIRE_MSC_VERSION "_x86_" FIRE_ADDRESS_WIDTH "s" FIRE_RUNTIME_FLAG FIRE_DEBUG_FLAG ".lib"
#define ZLIB_NAME "zlib_1_2_6_" FIRE_MSC_VERSION "_x86_" FIRE_ADDRESS_WIDTH "s" FIRE_RUNTIME_FLAG FIRE_DEBUG_FLAG ".lib"
#endif

/* Only the fire library will define FIRE_EXPORTS */
#ifdef FIRE_EXPORTS
#ifdef _USRDLL
#define RT_API __declspec(dllexport)
#ifndef RT_TOOLKIT
#define FIRE_API __declspec(dllexport)
#else
// Only export RT_API classes when building the Datafeed toolkit
#define FIRE_API
#endif
#ifndef FIRE_MSWIN_NO_AUTOLINK
#pragma comment(lib, OPENSSL_LIBEAY_NAME)
#pragma comment(lib, OPENSSL_SSLEAY_NAME)
#pragma comment(lib, CURL_LIB_NAME)
#pragma comment(lib, ZLIB_NAME)
#pragma comment(lib, CRYPTOPP_LIB_NAME)
#endif
#else
/* So we can build a static lib as well */
#define RT_API
#define FIRE_API
#endif

/* Client applications and/or DLL's (will not have FIRE_EXPORTS defined) */
/* There is no generic way for the client to specify a static library    */
/* Therefore make clients define FIRE_LINK_STATIC.                       */
#else
#ifndef FIRE_MSWIN_NO_AUTOLINK
#pragma comment(lib, FIRE_LIB_NAME)
#endif
#ifndef FIRE_LINK_STATIC
#define RT_API __declspec(dllimport)
#define FIRE_API __declspec(dllimport)
#else
#define RT_API
#define FIRE_API
#ifndef FIRE_MSWIN_NO_AUTOLINK
#pragma comment(lib, OPENSSL_LIBEAY_NAME)
#pragma comment(lib, OPENSSL_SSLEAY_NAME)
#pragma comment(lib, CURL_LIB_NAME)
#pragma comment(lib, ZLIB_NAME)
#pragma comment(lib, "Ws2_32.lib")
#pragma comment(lib, CRYPTOPP_LIB_NAME)
#endif
#endif
#endif
