// =====================================================================================
//
//       Filename:  otp_demo.cpp
//
//    Description:  Short example to demonstrate One Time Password(OTP)
//
//         Author:  Factset Datafeed Engineers
//        Version:  1.0
//
// Copyright (c) 2017, FactSet Research Systems Inc
// =====================================================================================

#include <iostream>
#include <FDS/rt_api.h>
#include <FDS/rt_fields.h>

// Edit the constants below to reflect your setup
const char* SERVICE = "FDS1";
const char*    HOST = "api-stage.df.factset.com";


static void on_message(int tag, const FDS::RT_Message *msg, const FDS::RT_Record *rec, void *closure)
  {
     FDS::RT_Field field = msg->get(FDS::FIDS::MSG_TYPE);
     std::cout << "Message Type " << field.to_string() << std::endl;
     std::cout << std::endl;

     std::cout << "RT Message" << std::endl;
     std::cout << msg->get_key() << "\n" << *msg << std::endl;
     std::cout << std::endl;

     std::cout << "RT Record" << std::endl;
     std::cout << *rec << std::endl;
     std::cout << std::endl;
  }

int main(int argc, char *argv[])
{
   if(argc < 3)
   {
      std::cerr << "Usage: "
                << argv[0]
                << " <user-serial>"
                << " <full path to key-counter file>"
                << std::endl;
      return 1;
   }

   std::string user_serial(argv[1]);
   std::string full_path(argv[2]);

   // Removing the folder location from full path
   size_t sep_loc = full_path.find_last_of("/");
   std::string path = "";
   std::string device_id = "";

   if(sep_loc != std::string::npos)
   {
      path = full_path.substr(0, sep_loc);
      device_id = full_path.substr(sep_loc+1);
   }
   else
   {
      device_id = full_path;
   }

   // Removing .data extension from the key-counter data file
   size_t ext_loc = device_id.find(".data\0");
   if(ext_loc != std::string::npos)
   {
      device_id = device_id.substr(0, ext_loc);
   }

   // Setting up RT_Consumer object for contacting FACTSET servers
   FDS::RT_Consumer consumer;
   consumer.set_connection_info( HOST,
                                 user_serial.c_str(),
                                 device_id.c_str(),
                                 NULL,
                                 NULL,
                                 path.c_str(),
                                 false
                              );

   //Initiate connection to FACTSET servers using the configuration setup
   std::cout << "\nConnecting to host: \"" << HOST << "\"...";
   consumer.connect();
   std::cout << "\tdone" <<std::endl;

   // Begin Session
   std::cout << "Subscribing to \'AAPL\'...";
   FDS::RT_Request req1(SERVICE, "AAPL");
   int tag1 = 0;
   consumer.request(req1, on_message, NULL, &tag1);
   std::cout << "done" <<std::endl;

   std::cout << "Subscribing to \'GOOG\'...";
   FDS::RT_Request req2(SERVICE, "GOOG");
   int tag2 = 0;
   consumer.request(req2, on_message, NULL, &tag2);
   std::cout << "done" <<std::endl;

   std::cout << "Subscribing to \'NVDA\'...";
   FDS::RT_Request req3(SERVICE, "NVDA");
   int tag3 = 0;
   consumer.request(req3, on_message, NULL, &tag3);
   std::cout << "done" <<std::endl;

   while(true)
   {
      consumer.dispatch(-1);
   }
   return 0;
}
